#include <windows.h>
#include <winbase.h>
#include <time.h>
#include <stdio.h>
#include <math.h>
#include <sys/types.h>

#define GBE_1000BASE_T_ACTIVE    1.217    /* in Watts */
#define GBE_0BASE_X        0.065    /* in Watts */

double d_energy = 0.0;        /* in joules ... power x time */
double d_ontime = 0.0;        /* amount of time spent transmitting data */
double d_time = 0.0;
double d_delaytime = 0.0;    /* cumulative amount of time packets were delayed due to powerup */
unsigned u_stalledpktcnt = 0;    /* absolute count of the packets delayed -or- count of number of idle transitions */
double d_waketime        = 0.000010; /* recovery time in seconds - (10) usec */
double d_sleeptime        = 0.000001; /* suspend time in seconds - (10) usec */
double d_stalltime        = 0.000010; /* threshold to wait for next packet before suspending */

int    main( int argc, char  *argv[] ) {
    int      packets = 0;
    double   t_start, t_end, t_next;
    double   pkt_tx, pkt_idle, pkt_idle_adjust;
    double   d_backlog        = 0.000000; /* backlog of latency which can be hidden by overlapping transmits ... */
    double   toggle = 0.0;
    int      initialized = 0;
    int      fields_processed;
    char     inputline[128];
    FILE     *inp_file;
    /*
     * format expected in the file is output from an Opnet simulation trace of traffic
     * which reports 
     * [time , as a floating point value in seconds]  [0.0 | 0.1 , for off/on pulse]
     * ex: 
     * 110.005000000000    1.000000000000        
     * 110.005000576000    0.000000000000    
     */

    if (argc < 2) {
        printf("usage:%s filename [stall in sec], where filename is an Opnet trace file\n", argv[0]);
        exit(1);
    }

    if (argc > 2) 
        d_stalltime    = (double)atof(argv[2]);

    inp_file = fopen(argv[1], "r");

    if (NULL == inp_file) {
    
        printf("failed to open %s (%s)\n", argv[1], strerror(errno));
        exit(1);
    }

    while( fgets(inputline, 128, inp_file) != NULL) {
        /* parse the string, figure out packet start/end transitions */
    
        fields_processed = sscanf(inputline,"%lf    %lf", &t_next, &toggle);
    
        if (fields_processed <= 0)
            continue;
    
        if ((1.0 == toggle) && (0 == initialized)) {
            /* latch t_start, continue to find next rising edge */
            t_start = t_next;
            initialized = 1;
            continue;
        }
    
        if (0.0 == toggle) {
            t_end = t_next;
            d_ontime += (t_end - t_start);
            continue;
        }
    
        /* process the power consumed until the edge of the next packet */
    
        packets++;

	pkt_tx = t_end - t_start;
	pkt_idle = t_next - t_end;

        /* 
	 * we only count the "current" on->off cycle 
	 *
         * algorithm : 
         * if the next event "soon" then
         *     assume we leave the unit on
         *     increment d_energy by GBE_1000BASE_T_ACTIVE * NetEventTimeInterval
         * else the next event is far enough in the future 
         *     problem case (a) the time in the future is beyond "soon" but before we have 
         *                      completely powered down the unit - interrupt power down and start power up
         *     problem case (b) a packet starts a wakeup, then another follows but with delay < wakeup time ...
         *                      in actuality the phy would complete wakeup and burst out the two frames
         *                      but we would count the power of the delay which is already counted in the
         *                      wakeup powerup time ...track the hidden latency and don't add the phantom power
         *
         *     increment d_delaytime by d_waketime
         *     increment d_stalledpktcnt 
         *     increment d_energy by (d_sleeptime * GBE_1000BASE_T_ACTIVE)
         *     increment d_energy by (NextEventTimeInterval * GBE_0BASE_X);
         *     increment d_energy by (d_waketime * GBE_1000BASE_T_ACTIVE)
         * increment d_time by NextEventTimeInterval
         */ 

	/* in all cases we use full power to transmit the packet */
        d_energy += GBE_1000BASE_T_ACTIVE * pkt_tx;

        if (d_backlog < 0.0) 
            d_backlog = 0.0;

	/* 
	 * we may be handling a backlog of packets which were delayed by 
	 * a previous wakeup recovery time 
	 */
	if (pkt_idle > d_backlog)
	    pkt_idle_adjust = pkt_idle - d_backlog;
	else
	    pkt_idle_adjust = 0; /* backlog consumes real inter-packet idle time */

        if (pkt_idle_adjust < d_stalltime)
	{
            d_energy += GBE_1000BASE_T_ACTIVE * pkt_idle; /* the full idle time, not the adjusted idle */
            d_backlog -= pkt_idle;
        }
        else if (pkt_idle_adjust < (d_waketime + d_sleeptime)) 
	{
	    /* 
	     * worst case - timing is just right to bounce the logic
	     * introducing latency as well as increasing power/bit.
	     * note: if you set stalltime = 0 from the command line,
	     * this code will always be executed, showing erroneous 
	     * wattage because the wattage is calculated assuming 
	     * minimal delays introduced to the packet transmittion
	     * due to this algorithm. With stall=0, we introduce 
	     * sleeptime + wakeup time to every packet.
	     */
            d_delaytime += (d_sleeptime + d_waketime);
            u_stalledpktcnt++;
            d_energy += (GBE_1000BASE_T_ACTIVE * d_sleeptime);
            d_energy += (GBE_1000BASE_T_ACTIVE * d_waketime);
            d_backlog += (d_sleeptime + d_waketime - pkt_idle_adjust);
        } 
        else 
	{
            d_delaytime += d_waketime;
            u_stalledpktcnt++;
            d_energy += (GBE_1000BASE_T_ACTIVE * d_sleeptime);
            d_energy += (GBE_0BASE_X * pkt_idle_adjust);
            d_energy += (GBE_1000BASE_T_ACTIVE * d_waketime);
            d_backlog += (d_waketime);
        }
        d_time += (pkt_tx + pkt_idle);

        /* latch t_start for next rising edge */
        t_start = t_next;
    }

    fclose(inp_file);
    printf("packets=%d ", packets);
    printf("time=%f ", (float)d_time );
    printf("delay=%f ", (float)d_delaytime);
    printf("actual time=%f ", (float)(d_time + d_delaytime) );
    printf("total_energy=%f ", (float)d_energy);
    printf("stalls=%d ", u_stalledpktcnt);
    printf("avg pwr=%f ", (d_energy/(d_time + d_delaytime)) * 1000);
    printf("pkts/sec=%f ", ((double)(packets)/d_time) ) ;
    printf("util=%f %%", ((double)(d_ontime)/d_time) * 100.0 ) ;

    return(0);
}


